@@ -30,31 +30,25 @@
 
 (require 'haskell-mode)
 (require 'haskell-compile)
-(require 'haskell-navigate-imports)
 (require 'haskell-process)
-(require 'haskell-collapse)
 (require 'haskell-session)
 (require 'haskell-font-lock)
 (require 'haskell-presentation-mode)
 (require 'haskell-utils)
-
+(require 'haskell-string)
 (require 'ansi-color)
 (require 'cl-lib)
 (require 'etags)
 
-(defvar haskell-interactive-mode-history-index)
-(make-variable-buffer-local 'haskell-interactive-mode-history-index)
+(defvar-local haskell-interactive-mode-history-index 0)
 
-(defvar haskell-interactive-mode-history (list))
-(make-variable-buffer-local 'haskell-interactive-mode-history)
+(defvar-local haskell-interactive-mode-history (list))
 
-(defvar haskell-interactive-mode-old-prompt-start
-  nil
+(defvar-local haskell-interactive-mode-old-prompt-start nil
   "Mark used for the old beginning of the prompt.")
-(make-variable-buffer-local 'haskell-interactive-mode-old-prompt-start)
 
 (defun haskell-interactive-prompt-regex ()
-  "Generate a regex for searching for any occurence of the prompt\
+  "Generate a regex for searching for any occurrence of the prompt\
 at the beginning of the line.  This should prevent any
 interference with prompts that look like haskell expressions."
   (concat "^" (regexp-quote haskell-interactive-prompt)))
@@ -62,12 +56,14 @@ interference with prompts that look like haskell expressions."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Globals used internally
 
+(declare-function haskell-interactive-kill "haskell")
+
 (defvar haskell-interactive-mode-map
   (let ((map (make-sparse-keymap)))
     (define-key map (kbd "RET") 'haskell-interactive-mode-return)
     (define-key map (kbd "SPC") 'haskell-interactive-mode-space)
     (define-key map (kbd "C-j") 'haskell-interactive-mode-newline-indent)
-    (define-key map (kbd "C-a") 'haskell-interactive-mode-beginning)
+    (define-key map [remap move-beginning-of-line] 'haskell-interactive-mode-bol)
     (define-key map (kbd "<home>") 'haskell-interactive-mode-beginning)
     (define-key map (kbd "C-c C-k") 'haskell-interactive-mode-clear)
     (define-key map (kbd "C-c C-c") 'haskell-process-interrupt)
@@ -87,12 +83,11 @@ interference with prompts that look like haskell expressions."
 (define-derived-mode haskell-interactive-mode fundamental-mode "Interactive-Haskell"
   "Interactive mode for Haskell.
 
-See Info node `(haskell-mode)haskell-interactive-mode' for more
-information.
-
 Key bindings:
 \\{haskell-interactive-mode-map}"
   :group 'haskell-interactive
+  :syntax-table haskell-mode-syntax-table
+
   (setq haskell-interactive-mode-history (list))
   (setq haskell-interactive-mode-history-index 0)
 
@@ -109,13 +104,12 @@ Key bindings:
 (defvar haskell-interactive-mode-result-end
   nil
   "Mark used to figure out where the end of the current result output is.
-Used to distinguish betwen user input.")
+Used to distinguish between user input.")
 
-(defvar haskell-interactive-previous-buffer nil
+(defvar-local haskell-interactive-previous-buffer nil
   "Records the buffer to which `haskell-interactive-switch-back' should jump.
 This is set by `haskell-interactive-switch', and should otherwise
 be nil.")
-(make-variable-buffer-local 'haskell-interactive-previous-buffer)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Hooks
@@ -135,6 +129,15 @@ be nil.")
   "Face for the prompt."
   :group 'haskell-interactive)
 
+;;;###autoload
+(defface haskell-interactive-face-prompt-cont
+  '((t :inherit font-lock-keyword-face))
+  "Face for GHCi's prompt-cont in multi-line mode."
+  :group 'haskell-interactive)
+
+;;;###autoload
+(define-obsolete-face-alias 'haskell-interactive-face-prompt2 'haskell-interactive-face-prompt-cont "16.2")
+
 ;;;###autoload
 (defface haskell-interactive-face-compile-error
   '((t :inherit compilation-error))
@@ -166,7 +169,8 @@ be nil.")
   "Make newline and indent."
   (interactive)
   (newline)
-  (indent-according-to-mode))
+  (indent-to (length haskell-interactive-prompt))
+  (indent-relative))
 
 (defun haskell-interactive-mode-kill-whole-line ()
   "Kill the whole REPL line."
@@ -205,13 +209,24 @@ be nil.")
         (next-error-no-select 0)
       (self-insert-command n))))
 
-(defun haskell-interactive-at-prompt ()
-  "If at prompt, return start position of user-input, otherwise return nil."
-  (if (>= (point)
+(defun haskell-interactive-at-prompt (&optional end-line)
+  "If at prompt, return start position of user-input, otherwise return nil.
+If END-LINE is non-nil, then return non-nil when the end of line
+is at the prompt."
+  (if (>= (if end-line (line-end-position) (point))
           haskell-interactive-mode-prompt-start)
       haskell-interactive-mode-prompt-start
     nil))
 
+(defun haskell-interactive-mode-bol ()
+  "Go to beginning of current line, but after current prompt if any."
+  (interactive)
+  (let ((beg (line-beginning-position))
+        (end (line-end-position)))
+    (goto-char (if (>= end haskell-interactive-mode-prompt-start beg)
+                   haskell-interactive-mode-prompt-start
+                 beg))))
+
 (define-derived-mode haskell-error-mode
   special-mode "Error"
   "Major mode for viewing Haskell compile errors.")
@@ -235,22 +250,6 @@ be nil.")
                                   (point-min))
                              end))))))))
 
-(defun haskell-interactive-mode-cleanup-response (expr response)
-  "Ignore the mess that GHCi outputs on multi-line input."
-  (if (not (string-match "\n" expr))
-      response
-    (let ((i 0)
-          (out "")
-          (lines (length (split-string expr "\n"))))
-      (cl-loop for part in (split-string response "| ")
-               do (setq out
-                        (concat out
-                                (if (> i lines)
-                                    (concat (if (or (= i 0) (= i (1+ lines))) "" "| ") part)
-                                  "")))
-               do (setq i (1+ i)))
-      out)))
-
 (defun haskell-interactive-mode-multi-line (expr)
   "If a multi-line expression EXPR has been entered, then reformat it to be:
 
@@ -259,30 +258,18 @@ do the
    multi-liner
    expr
 :}"
-  (if (not (string-match "\n" expr))
+  (if (not (string-match-p "\n" expr))
       expr
-    (let* ((i 0)
-           (lines (split-string expr "\n"))
-           (len (length lines)))
-      (mapconcat 'identity
-                 (cl-loop for line in lines
-                          collect (cond ((= i 0)
-                                         (concat ":{" "\n" line))
-                                        ((= i (1- len))
-                                         (concat line "\n" ":}"))
-                                        (t
-                                         line))
-                          do (setq i (1+ i)))
-                 "\n"))))
-
-(defun haskell-interactive-trim (line)
-  "Trim indentation off of LINE in the REPL."
-  (if (and (string-match "^[ ]+" line)
-           (> (length line)
-              (length haskell-interactive-prompt)))
-      (substring line
-                 (length haskell-interactive-prompt))
-    line))
+    (let ((pre (format "^%s" (regexp-quote haskell-interactive-prompt)))
+          (lines (split-string expr "\n")))
+      (cl-loop for elt on (cdr lines) do
+               (setcar elt (replace-regexp-in-string pre "" (car elt))))
+      ;; Temporarily set prompt2 to be empty to avoid unwanted output
+      (concat ":set prompt2 \"\"\n"
+              ":{\n"
+              (mapconcat #'identity lines "\n")
+              "\n:}\n"
+              (format ":set prompt-cont \"%s\"" haskell-interactive-prompt-cont)))))
 
 (defun haskell-interactive-mode-line-is-query (line)
   "Is LINE actually a :t/:k/:i?"
@@ -316,50 +303,54 @@ SESSION, otherwise operate on the current buffer."
   (with-current-buffer (if session
                            (haskell-session-interactive-buffer session)
                          (current-buffer))
-    (goto-char (point-max))
-    (let ((prompt (propertize haskell-interactive-prompt
-                              'font-lock-face 'haskell-interactive-face-prompt
-                              'prompt t
-                              'read-only t
-                              'rear-nonsticky t)))
-      ;; At the time of writing, front-stickying the first char gives an error
-      ;; Has unfortunate side-effect of being able to insert before the prompt
-      (insert (substring prompt 0 1)
-              (propertize (substring prompt 1)
-                          'front-sticky t)))
-    (let ((marker (set (make-local-variable 'haskell-interactive-mode-prompt-start)
-                       (make-marker))))
-      (set-marker marker (point)))
-    (when haskell-interactive-mode-scroll-to-bottom
+    (save-excursion
+      (goto-char (point-max))
+      (let ((prompt (propertize haskell-interactive-prompt
+                                'font-lock-face 'haskell-interactive-face-prompt
+                                'prompt t
+                                'read-only haskell-interactive-prompt-read-only
+                                'rear-nonsticky t)))
+        ;; At the time of writing, front-stickying the first char gives an error
+        ;; Has unfortunate side-effect of being able to insert before the prompt
+        (insert (substring prompt 0 1)
+                (propertize (substring prompt 1)
+                            'front-sticky t)))
+      (let ((marker (setq-local haskell-interactive-mode-prompt-start (make-marker))))
+        (set-marker marker (point))))
+    (when (haskell-interactive-at-prompt t)
       (haskell-interactive-mode-scroll-to-bottom))))
 
 (defun haskell-interactive-mode-eval-result (session text)
   "Insert the result of an eval as plain text."
   (with-current-buffer (haskell-session-interactive-buffer session)
-    (goto-char (point-max))
-    (insert (ansi-color-apply
-             (propertize text
-                         'font-lock-face 'haskell-interactive-face-result
-                         'front-sticky t
-                         'prompt t
-                         'read-only t
-                         'rear-nonsticky t
-                         'result t)))
-    (haskell-interactive-mode-handle-h)
-    (let ((marker (set (make-local-variable 'haskell-interactive-mode-result-end)
-                       (make-marker))))
-      (set-marker marker
-                  (point)
-                  (current-buffer)))
-    (when haskell-interactive-mode-scroll-to-bottom
-      (haskell-interactive-mode-scroll-to-bottom))))
+    (let ((at-end (eobp))
+          (prop-text (propertize text
+                                 'font-lock-face 'haskell-interactive-face-result
+                                 'front-sticky t
+                                 'prompt t
+                                 'read-only haskell-interactive-mode-read-only
+                                 'rear-nonsticky t
+                                 'result t)))
+      (save-excursion
+        (goto-char (point-max))
+        (when (string= text haskell-interactive-prompt-cont)
+          (setq prop-text
+                (propertize prop-text
+                            'font-lock-face 'haskell-interactive-face-prompt-cont
+                            'read-only haskell-interactive-prompt-read-only)))
+        (insert (ansi-color-apply prop-text))
+        (haskell-interactive-mode-handle-h)
+        (let ((marker (setq-local haskell-interactive-mode-result-end (make-marker))))
+          (set-marker marker (point))))
+      (when at-end
+        (haskell-interactive-mode-scroll-to-bottom)))))
 
 (defun haskell-interactive-mode-scroll-to-bottom ()
   "Scroll to bottom."
   (let ((w (get-buffer-window (current-buffer))))
     (when w
       (goto-char (point-max))
-      (set-window-point w (point-max)))))
+      (set-window-point w (point)))))
 
 (defun haskell-interactive-mode-compile-error (session message)
   "Echo an error."
@@ -384,7 +375,7 @@ SESSION, otherwise operate on the current buffer."
                                   'expandable t
                                   'font-lock-face type
                                   'front-sticky t
-                                  'read-only t
+                                  'read-only haskell-interactive-mode-read-only
                                   'rear-nonsticky t))
               (insert (propertize (concat (match-string 2 message) "\n")
                                   'collapsible t
@@ -392,12 +383,12 @@ SESSION, otherwise operate on the current buffer."
                                   'front-sticky t
                                   'invisible haskell-interactive-mode-hide-multi-line-errors
                                   'message-length (length (match-string 2 message))
-                                  'read-only t
+                                  'read-only haskell-interactive-mode-read-only
                                   'rear-nonsticky t)))
           (insert (propertize (concat message "\n")
                               'font-lock-face type
                               'front-sticky t
-                              'read-only t
+                              'read-only haskell-interactive-mode-read-only
                               'rear-nonsticky t)))))))
 
 (defun haskell-interactive-mode-insert (session message)
@@ -514,18 +505,30 @@ FILE-NAME only."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Misc
 
+(declare-function haskell-interactive-switch "haskell")
+(declare-function haskell-session "haskell")
+
 (defun haskell-session-interactive-buffer (s)
   "Get the session interactive buffer."
   (let ((buffer (haskell-session-get s 'interactive-buffer)))
     (if (and buffer (buffer-live-p buffer))
         buffer
-      (let ((buffer (get-buffer-create (format "*%s*" (haskell-session-name s)))))
-        (haskell-session-set-interactive-buffer s buffer)
-        (with-current-buffer buffer
-          (haskell-interactive-mode)
-          (haskell-session-assign s))
-        (haskell-interactive-switch)
-        buffer))))
+      (let ((buffer-name (format "*%s*" (haskell-session-name s)))
+            (index 0))
+        (while (get-buffer buffer-name)
+          (setq buffer-name (format "*%s <%d>*" (haskell-session-name s) index))
+          (setq index (1+ index)))
+        (let ((buffer (get-buffer-create buffer-name)))
+          (haskell-session-set-interactive-buffer s buffer)
+          (with-current-buffer buffer
+            (haskell-interactive-mode)
+            (haskell-session-assign s))
+          (haskell-interactive-switch)
+          buffer)))))
+
+(defun haskell-interactive-buffer ()
+  "Get the interactive buffer of the session."
+  (haskell-session-interactive-buffer (haskell-session)))
 
 (defun haskell-process-cabal-live (state buffer)
   "Do live updates for Cabal processes."
@@ -555,7 +558,7 @@ FILE-NAME only."
     span))
 
 (defun haskell-process-suggest-add-package (session msg)
-  "Add tthe (matched) module to your cabal file.
+  "Add the (matched) module to your cabal file.
 Cabal file is selected using SESSION's name, module matching is done in MSG."
   (let* ((suggested-package (match-string 1 msg))
          (package-name (replace-regexp-in-string "-[^-]+$" "" suggested-package))
@@ -566,11 +569,11 @@ Cabal file is selected using SESSION's name, module matching is done in MSG."
     (haskell-mode-toggle-interactive-prompt-state)
     (unwind-protect
         (when (y-or-n-p
-               (format "Add `%s' to %s?"
+               (format "Add `%s' to %s? "
                        package-name
                        cabal-file))
           (haskell-cabal-add-dependency package-name version nil t)
-          (when (y-or-n-p (format "Enable -package %s in the GHCi session?" package-name))
+          (when (y-or-n-p (format "Enable -package %s in the GHCi session? " package-name))
             (haskell-process-queue-without-filters
              (haskell-session-process session)
              (format ":set -package %s" package-name))))
@@ -775,7 +778,7 @@ Will automatically import it qualified as Present."
                           (mapconcat 'identity (mapcar 'number-to-string id) ",")
                           hash)))
            (reply
-            (if (string-match "^*** " text)
+            (if (string-prefix-p "*** " text)
                 '((rep nil))
               (read text))))
       ;; Not necessary, but nice to restore it to the expression that
@@ -1036,7 +1039,7 @@ This completion function is used in interactive REPL buffer itself."
 If there is one, pop that up in a buffer, similar to `debug-on-error'."
   (when (and haskell-interactive-types-for-show-ambiguous
              (string-match "^\n<interactive>:[-0-9]+:[-0-9]+:" response)
-             (not (string-match "^\n<interactive>:[-0-9]+:[-0-9]+:[\n ]+Warning:" response)))
+             (not (string-match "^\n<interactive>:[-0-9]+:[-0-9]+:[\n ]+[Ww]arning:" response)))
     (let ((inhibit-read-only t))
       (delete-region haskell-interactive-mode-prompt-start (point))
       (set-marker haskell-interactive-mode-prompt-start
