@@ -85,14 +85,14 @@ buffer before presenting message."
     (with-current-buffer buffer
 
       (when (boundp 'shm-display-quarantine)
-        (set (make-local-variable 'shm-display-quarantine) nil))
+        (setq-local shm-display-quarantine nil))
 
       (when clear (haskell-presentation-clear))
       (haskell-session-assign session)
+      (goto-char (point-min))
+      (forward-line 2)
       (save-excursion
         (let ((buffer-read-only nil))
-          (goto-char (point-min))
-          (forward-line 2)
           (insert code "\n\n"))))
 
     (if (eq major-mode 'haskell-presentation-mode)
